﻿using Microsoft.AspNetCore.WebUtilities;
using MimeKit;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;

namespace Pop3ToADefHelpDesk
{
    public class CreateTicket
    {
        public static async Task<DTOStatus> CreateADefHelpDeskTicket(
            DTOTask paramDTOTask,
            DTOTaskDetail paramDTOTaskDetail,
            MimeMessage paramMimeMessage,
            string APIWebAddress,
            string paramBearerToken)
        {
            // Final response
            HttpResponseMessage response;

            // The contents of the file will be stored in the MemoryStream
            MemoryStream stream = new MemoryStream();

            // Initialize classes and variables
            DTOStatus Result = new DTOStatus();
            HttpClient client = new HttpClient();
            string strFileName = "";

            using (client)
            {
                using (var request = new HttpRequestMessage())
                {
                    client.DefaultRequestHeaders.Authorization =
                        new AuthenticationHeaderValue("Bearer", paramBearerToken);

                    // Add Task parameters
                    // Call the ToKeyValue extension method
                    // to concatonate all the values into a single string
                    var TaskParameters = paramDTOTask.ToKeyValue();

                    // If there are Task Details add them
                    if (paramDTOTaskDetail != null)
                    {
                        // Call the ToKeyValue extension method
                        var TaskDetailParameters = paramDTOTaskDetail.ToKeyValue();

                        foreach (var item in TaskDetailParameters)
                        {
                            // Ass the TaskDetail parameters
                            TaskParameters.Add(item);
                        }
                    }

                    // The destination is the CreateTask API method
                    request.RequestUri =
                        new Uri(QueryHelpers.AddQueryString(
                            $"{APIWebAddress}api/V1/CreateTask"
                            , TaskParameters));

                    // The API method requires the 'verb' 
                    // or 'action' to be a 'Post'
                    request.Method = HttpMethod.Post;

                    // Determine the Email type
                    if (paramMimeMessage.Body.ContentType.MimeType == "text/html")
                    {
                        // This is a "text/html" file

                        // Get Email contents
                        var EmailContent = 
                            ((MimeKit.TextPart)paramMimeMessage.Body).Text;

                        // Save the Message to a stream 
                        byte[] encodedText = Encoding.Unicode.GetBytes(EmailContent);
                        stream.Write(encodedText, 0, encodedText.Length);
                        stream.Position = 0;

                        strFileName = "Attachment.html";
                    }
                    else
                    {
                        // If the email is not of type "text/html"
                        // Save the entire email as a .EML file
                        // ADefHelpDesk has the ability to view a EML file 
                        // and any attachments

                        // Save the .EML Message to a stream                    
                        paramMimeMessage.WriteTo(stream);
                        stream.Position = 0;

                        strFileName = "Attachment.EML";
                    }

                    using (stream)
                    {
                        MultipartFormDataContent _multiPartContent =
                            new MultipartFormDataContent();

                        // Serialize Request
                        StreamContent _fileData =
                            new StreamContent(stream);

                        _fileData.Headers.ContentType =
                            new MediaTypeHeaderValue("application/octet-stream");

                        ContentDispositionHeaderValue _contentDispositionHeaderValue =
                            new ContentDispositionHeaderValue("form-data");

                        _contentDispositionHeaderValue.Name = 
                            "objFile";
                        _contentDispositionHeaderValue.FileName = 
                            strFileName;
                        _fileData.Headers.ContentDisposition = 
                            _contentDispositionHeaderValue;

                        _multiPartContent.Add(_fileData, "objFile");

                        // Set content
                        request.Content = _multiPartContent;

                        // Send request
                        response = client.SendAsync(request).Result;
                    }

                    // If the Bearer token is expired
                    if (response.StatusCode == System.Net.HttpStatusCode.Unauthorized)
                    {
                        Result.Success = false;
                        Result.StatusMessage = "Unauthorized";
                        return Result;
                    }

                    // ADefHelpDesk will respond with JSON
                    var JsonDataResponse =
                        await response.Content.ReadAsStringAsync();

                    // The final result is the Deserialized response
                    Result =
                        JsonConvert.DeserializeObject<DTOStatus>(JsonDataResponse);
                }
            }

            return Result;
        }
    }
}