﻿using Newtonsoft.Json;
using System;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;

namespace Pop3ToADefHelpDesk
{
    public class GetAuthToken
    {
        public static async Task<string> GetAuthTokenFromADefHelpDesk(
            string APIWebAddress, 
            DTOApiToken paramApiToken)
        {
            // Store the final result
            string strResult = "";

            HttpClient client = new HttpClient();

            // Use the HttpClient 
            using (client)
            {
                // Create a new REST request
                using (var request = new HttpRequestMessage())
                {
                    // The Swagger page indicates this must be a "Post"
                    request.Method = HttpMethod.Post;
                    // Set the destination to the method indicated 
                    // on the Swagger page
                    // to: api/V1/GetAuthToken
                    request.RequestUri = 
                        new Uri($"{APIWebAddress}api/V1/GetAuthToken");
                    // Convert the parameters to 
                    // JavaScript Object Notation (JSON) format 
                    var json = JsonConvert.SerializeObject(paramApiToken);
                    request.Content = 
                        new StringContent(json, Encoding.UTF8, "application/json");

                    // Make the request to the API endpoint on the ADefHelpDesk site
                    var response = client.SendAsync(request).Result;

                    // Receive the response
                    var JsonDataResponse =
                        await response.Content.ReadAsStringAsync();

                    // Convert the response (the JWT (Auth Token)) to a String value
                    strResult =
                        JsonConvert.DeserializeObject<string>(JsonDataResponse);

                    // Strip the word Bearer from the token
                    strResult = strResult.Replace("Bearer ", " ").Trim();
                }
            }

            // Return the JWT
            return strResult;
        }
    }
}